@@ -15,10 +15,11 @@ from pyhap.const import HAP_REPR_AID, HAP_REPR_IID
 from .hap_crypto import HAPCrypto
 from .hap_event import create_hap_event
 from .hap_handler import HAPResponse, HAPServerHandler
+from .util import async_create_background_task
 
 logger = logging.getLogger(__name__)
 
-HIGH_WRITE_BUFFER_SIZE = 2 ** 19
+HIGH_WRITE_BUFFER_SIZE = 2**19
 # We timeout idle connections after 90 hours as we must
 # clean up unused sockets periodically. 90 hours was choosen
 # as its the longest time we expect a user to be away from
@@ -48,7 +49,7 @@ class HAPServerProtocol(asyncio.Protocol):
         self.last_activity = None
         self.hap_crypto = None
         self._event_timer = None
-        self._event_queue = []
+        self._event_queue = {}
 
         self.start_time = None
 
@@ -112,7 +113,7 @@ class HAPServerProtocol(asyncio.Protocol):
 
     def queue_event(self, data: dict, immediate: bool) -> None:
         """Queue an event for sending."""
-        self._event_queue.append(data)
+        self._event_queue[(data[HAP_REPR_AID], data[HAP_REPR_IID])] = data
         if immediate:
             self.loop.call_soon(self._send_events)
         elif not self._event_timer:
@@ -212,14 +213,14 @@ class HAPServerProtocol(asyncio.Protocol):
         subscribed_events = self._event_queue_with_active_subscriptions()
         if subscribed_events:
             self.write(create_hap_event(subscribed_events))
-        self._event_queue = []
+        self._event_queue.clear()
 
     def _event_queue_with_active_subscriptions(self):
         """Remove any topics that have been unsubscribed after the event was generated."""
         topics = self.accessory_driver.topics
         return [
             event
-            for event in self._event_queue
+            for event in self._event_queue.values()
             if self.peername
             in topics.get(get_topic(event[HAP_REPR_AID], event[HAP_REPR_IID]), [])
         ]
@@ -253,7 +254,7 @@ class HAPServerProtocol(asyncio.Protocol):
             self.request_body = None
             return True
 
-        return self._handle_invalid_conn_state("Unexpected event: {}".format(event))
+        return self._handle_invalid_conn_state(f"Unexpected event: {event}")
 
     def _process_response(self, response) -> None:
         """Process a response from the handler."""
@@ -270,7 +271,7 @@ class HAPServerProtocol(asyncio.Protocol):
             self.hap_crypto = HAPCrypto(response.shared_key)
         # Only update mDNS after sending the response
         if response.pairing_changed:
-            asyncio.ensure_future(
+            async_create_background_task(
                 self.loop.run_in_executor(None, self.accessory_driver.finish_pair)
             )
 
